Module migration

Module migration 

Types to declare schemas and migrations.

A good starting point is too look at crate::migration::schema.

Structs

Config
Config is used to open a database from a file or in memory.
Migrated
Migrated provides a proof of migration.
MigratedOptional
Proof that a table is at least partially migrated.
Migrator
Migrator is used to apply database migrations.
TransactionMigrate
Transaction type for use in migrations.

Enums

ForeignKeys
Which method should be used to check foreign-key constraints.
Synchronous
https://www.sqlite.org/pragma.html#pragma_synchronous

Functions

hash_schemadev
Calculate the hash of a shema.

Attribute Macros

schema
Multi-versioned schema definition macro.