MigratedOptional

Struct MigratedOptional 

pub struct MigratedOptional<T: Migrateable> { /* private fields */ }
Proof that a table is at least partially migrated.

This type can be turned into Migrated by providing an error handler.

Implementations

Source

impl<T: Migrateable> MigratedOptional<T>

pub fn map_fk_err<'t>( self, f: impl 't + FnOnce() -> Infallible, ) -> Migrated<'t, T>

The closure is called when there is a foreign key error due to some row being removed.

Source

impl<T: Migrateable<Referer = Infallible>> MigratedOptional<T>

pub fn no_reference(self) -> Migrated<'static, T>

The table has the #[no_reference] attribute, so partial migration is always ok.