pub struct TableRow<T: Table> { /* private fields */ }Expand description
Row reference that can be used in any query in the same transaction.
TableRow is restricted to a single thread to prevent it from being used in a different transaction.