pub struct MigratedOptional<T: Migrateable> { /* private fields */ }Expand description
Proof that a table is at least partially migrated.
This type can be turned into Migrated by providing an error handler.
Implementations
Sourceimpl<T: Migrateable> MigratedOptional<T>
impl<T: Migrateable> MigratedOptional<T>
Sourcepub fn map_fk_err<'t>(
self,
f: impl 't + FnOnce() -> Infallible,
) -> Migrated<'t, T>
pub fn map_fk_err<'t>( self, f: impl 't + FnOnce() -> Infallible, ) -> Migrated<'t, T>
The closure is called when there is a foreign key error due to some row being removed.
Sourceimpl<T: Migrateable<Referer = Infallible>> MigratedOptional<T>
impl<T: Migrateable<Referer = Infallible>> MigratedOptional<T>
Sourcepub fn no_reference(self) -> Migrated<'static, T>
pub fn no_reference(self) -> Migrated<'static, T>
The table has the #[no_reference] attribute, so partial migration is always ok.