Conflict

Struct Conflict 

pub struct Conflict<T: Table> { /* private fields */ }
Error type that is used by crate::Transaction::insert and crate::Mutable::unique when there are at least two unique constraints.

The source of the error is the message received from sqlite. It contains the column names that were conflicted.