Mutable

Struct Mutable 

pub struct Mutable<'transaction, T: Table> { /* private fields */ }
Mutable access to columns of a single table row.

The whole row is retrieved and can be inspected from Rust code. However, only rows that are not used in a #[unique] constraint can be updated directly by dereferencing Mutable.

To update columns with a unique constraint, you have to use Mutable::unique.

Implementations

Source

impl<'transaction, T: Table> Mutable<'transaction, T>

pub fn into_table_row(self) -> TableRow<T>

Turn the Mutable into a TableRow.

This will end the lifetime of the Mutable, which is useful since Mutable does not have a non lexical lifetime, because of the Drop impl.

If you do not need the TableRow, then it is also possible to just call drop.

pub fn unique<O>( &mut self, f: impl FnOnce(&mut <T::Mutable as Deref>::Target) -> O, ) -> Result<O, T::Conflict>

Update unique constraint columns.

When the update succeeds, this function returns Ok, when it fails it returns Err with one of three conflict types:

If any of the changes made inside the closure conflict with an existing row, then all changes made inside the closure are reverted.

If the closure panics, then all changes made inside the closure are also reverted. Applying those changes is not possible, as conflicts can not be reported if there is a panic.